#include "TranspositionTable.h"
#include "MoveGeneration.h"
#include "ChessGame.h"
#include <random>

using namespace chessy;

std::array<HashValue, 64 * 12>  zobrist::pieces;
std::array<HashValue, 16>       zobrist::castlingRights;
std::array<HashValue, 8>        zobrist::enPassant;
HashValue                       zobrist::turn;


HashValue zobrist::getHash(const ChessGame& cg)
{
    HashValue value = 0;
    const Board& b = cg.getBoard();
    // iterate over all piece types
    for (int i = 0; i < 12; i++) {
        Side side = i < 6 ? WHITE_SIDE : BLACK_SIDE;
        Bitboard t = b.getBitboards(side)[i % 6];
        for (auto pos : PositionSet{ t }) {
            value ^= get(PieceType(i % 6), side, pos);
        }
    }
    value ^= castlingRights[cg.getCastlingRights()];
    if (cg.getEnPassantIndex() != -1)
        value ^= enPassant[cg.getEnPassantIndex()];

    if (cg.getTurn() != WHITE_SIDE)
        value ^= turn;

    return value;
}


void zobrist::initialize(void)
{
    uint64_t seed = 0xCFC9725D8992B787;
    std::mt19937_64 engine(seed);
    std::uniform_int_distribution<HashValue> distribution;
    auto rnd = [&]() { return distribution(engine); };

    for (auto& v : pieces)
        v = rnd();
    for (auto& v : castlingRights)
        v = rnd();
    for (auto& v : enPassant)
        v = rnd();
    turn = rnd();
}
